<?php
ob_start();

define('OSCWARE_CONNECTOR', '2.41');

// Don't show errors by default so we don't interfere with the XML response.
// Our own error handler is not respecting the settings so everything is logged.
@error_reporting(0);
@ini_set('display_errors', 0);
@ini_set('memory_limit', -1);

define("cdatar", base64_decode("XV0+"));
define("cdatal", base64_decode("PCFbQ0RBVEFb"));
define("rep1", base64_decode("XV0="));
define("rep2", base64_decode("Pg=="));
define("MYSQL_CONNECTION_CHARSET", "utf8");
define("LOGFILE", "logfile.txt");
define("LOGFILE_MAX_SIZE", 1024 * 1024); // 1 MB

define("CONNECTFOWNER", fileowner(__FILE__));
define("CONNECTFGROUP", filegroup(__FILE__));
define('TEST_TREAT_WARNINGS_AS_ERRORS', isset($_SERVER['HTTP_X_TREAT_WARNINGS_AS_ERRORS']) ? true : false);

// Enabled by $oscdebug=true in oscware-configure.php or DEBUG XML parameter
$OSCDEBUG = false;

// For the shutdown handler to skip execution when the script reached the end
$DONE = false;

function error_handler($errno, $errstr, $errfile, $errline) {
    static $logfile = null;

    // Move file out of the way
    if(file_exists(LOGFILE) && filesize(LOGFILE) > LOGFILE_MAX_SIZE) {
        rename(LOGFILE, str_replace('.txt', '.old', LOGFILE));
    }

    // Create file handle and set default timezone once
    if($logfile === null) {
        date_default_timezone_set('UTC');
        $logfile = fopen(LOGFILE, 'a');
    }

    // Logfile was not created
    if(!$logfile) {
        return false;
    }

    $timestamp = date('c');
    switch ($errno) {
        case E_ERROR:
        case E_USER_ERROR:
            $line = "$timestamp ERROR: $errstr ($errfile:$errline)\n";
            break;
        case E_WARNING:
        case E_USER_WARNING:
            $line = "$timestamp WARNING: $errstr ($errfile:$errline)\n";
            break;
        case E_NOTICE:
        case E_USER_NOTICE:
            $line = "$timestamp NOTICE: $errstr ($errfile:$errline)\n";
            break;
        default:
            $line = "$timestamp UNKNOWN ($errno): $errstr ($errfile:$errline)\n";
            break;
    }

    fputs($logfile, $line);

    # For unit testing
    if (TEST_TREAT_WARNINGS_AS_ERRORS) {
        if ($errno == E_WARNING || $errno == E_NOTICE) {
            throw new ErrorException($line);
        }
    }

    // Execute PHP's error handler to handle messages in error_reporting()
    return false;
}

set_error_handler('error_handler');

/**
 * A shutdown handler to handle possible unexpected script aborts due to fatal
 * errors
 */
function shutdown_handler() {
    global $DONE;

    // DONE is true, when the script reached the end. Since the last error can
    // be a custom log message, it's done this way.
    if($DONE) {
        return;
    }

    $error = error_get_last();
    if($error) {
        error_handler($error['type'], 'FATAL - ' . $error['message'], $error['file'], $error['line']);
    }
}
register_shutdown_function('shutdown_handler');

function call_error($error_id, $suberror = false)
{
    global $fehlercodes, $OSCDEBUG;
    $fehlermeldung = ((isset($fehlercodes[$error_id])) ? $fehlercodes[$error_id] : $error_id);
    if (function_exists('error_get_last') && $OSCDEBUG) {
        if (($p = error_get_last()) !== null) {
            $suberror .= ($suberror != false) ? ' -- ' . implode('-', $p) : implode('-', $p);
        }
    }
    if ($suberror != false) {
        $fehlermeldung .= ' | ' . $suberror;
    }
    return array('Erfolgreich' => cdatal . '0' . cdatar, 'FehlerCode' => cdatal . $error_id . cdatar, 'FehlerMeldung' => cdatareplace($fehlermeldung));
}

function cdatareplace($content)
{
    if (!$content) {
        return $content;
    }

    return cdatal . str_replace(cdatar, rep1 . cdatar . cdatal . rep2, $content) . cdatar;
}

function call_success($inhalt = array())
{
    $ar = array('Erfolgreich' => cdatal . '1' . cdatar);
    if (is_array($inhalt) && count($inhalt) > 0) {
        unset($inhalt['Erfolgreich']);
        foreach ($inhalt as $key => $value) {
            $ar[$key] = cdatareplace($value);
        }
    }
    return $ar;
}


function build_xml($a)
{
    $b = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<ErgebnisListe>\n";
    foreach ($a as $c) {
        $b .= "<Ergebnis>\n<ID>" . cdatal . $c['ID'] . cdatar . "</ID>\n";
        foreach ($c['Antwort'] as $d => $e) {
            $b .= "<" . $d . ">" . $e . "</" . $d . ">\n";
        }
        $b .= "</Ergebnis>\n";
    }
    $b .= "</ErgebnisListe>\n";

    return $b;
}

$theindex = -1;
$depth = 0;
$xml_values = array();

function startElement($parser, $data)
{
    global $theindex, $xml_values, $depth;
    $theindex++;
    $depth++;
    $xml_values[$theindex] = array('tag' => $data, 'type' => 'open', 'level' => $depth);
    $currentarraystart = $data;

}

function endElement($parser, $data)
{
    global $theindex, $xml_values, $depth;
    if ($xml_values[$theindex]['tag'] == $data) {
        $xml_values[$theindex]['type'] = 'complete';
    } else {
        global $theindex, $xml_values, $depth;
        $theindex++;
        $xml_values[$theindex] = array('tag' => $data, 'type' => 'close', 'level' => $depth);
    }
    $depth--;
}

function cdata($parser, $data)
{
    global $theindex, $xml_values, $depth;

    if (!$data) {
        return;
    }

    $data = str_replace("\r", "", str_replace("\n", "", str_replace("\r\n", "", trim($data))));
    if (strlen($data) > 0) {
        if (isset($xml_values[$theindex]['value'])) {
            $xml_values[$theindex]['value'] .= ((substr($data, 0, 1) == "]") ? '' : "(linebreak)") . $data;
        } else {
            $xml_values[$theindex]['value'] = $data;
        }
    }
}

function xml2array($contents)
{
    global $xml_values;
    if (!$contents) {
        return array();
    }
    if (!function_exists('xml_parser_create')) {
        return array();
    }
    $parser = xml_parser_create('');
    xml_parser_set_option($parser, XML_OPTION_TARGET_ENCODING, "UTF-8");
    xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
    xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1);
    if (phpversion() < 5) {
        $contents = str_replace(cdatar . cdatal, "", $contents);
        xml_set_element_handler($parser, 'startElement', 'endElement');
        xml_set_character_data_handler($parser, "cdata");
        xml_parse($parser, $contents);
    } else {
        xml_parse_into_struct($parser, trim($contents), $xml_values);
    }
    xml_parser_free($parser);

    if (!$xml_values)
        return;

    $xml_array = array();
    $parents = array();
    $opened_tags = array();
    $arr = array();

    $current = &$xml_array;

    $repeated_tag_index = array();
    foreach ($xml_values as $data) {
        unset($attributes, $value);

        extract($data);

        $result = array();
        $attributes_data = array();

        if (isset($value) && !is_null($value)) {
            if ($tag == 'SQLAusdruck') {
                $result = str_replace("(linebreak)", "", $value);
            } else {
                $value = str_replace("(linebreak)", "\r\n", $value);
                $result = ($value == "[inigetup\r\n]" || $value == "[inisetup\r\n]") ? str_replace("\r\n", "", $value) : $value;
            }

        }
        if ($tag == 'Parameter' && !isset($value)) {
            $result = '';
        }
        if (isset($attributes)) {
            foreach ($attributes as $attr => $val) {
                $attributes_data[$attr] = $val;
            }
        }

        if ($type == "open") {
            $parent[$level - 1] = &$current;
            if (!is_array($current) or (!in_array($tag, array_keys($current)))) {
                $current[$tag] = $result;
                if ($attributes_data)
                    $current[$tag . '_attr'] = $attributes_data;
                $repeated_tag_index[$tag . '_' . $level] = 1;

                $current = &$current[$tag];

            } else {

                if (isset($current[$tag][0])) {
                    $current[$tag][$repeated_tag_index[$tag . '_' . $level]] = $result;
                    $repeated_tag_index[$tag . '_' . $level]++;
                } else {
                    $current[$tag] = array($current[$tag], $result);
                    $repeated_tag_index[$tag . '_' . $level] = 2;

                    if (isset($current[$tag . '_attr'])) {
                        $current[$tag]['0_attr'] = $current[$tag . '_attr'];
                        unset($current[$tag . '_attr']);
                    }

                }
                $last_item_index = $repeated_tag_index[$tag . '_' . $level] - 1;
                $current = &$current[$tag][$last_item_index];
            }

        } elseif ($type == "complete") {

            if (!isset($current[$tag])) {
                $current[$tag] = $result;
                $repeated_tag_index[$tag . '_' . $level] = 1;
                if ($attributes_data)
                    $current[$tag . '_attr'] = $attributes_data;

            } else {
                if (isset($current[$tag][0]) and is_array($current[$tag])) {

                    $current[$tag][$repeated_tag_index[$tag . '_' . $level]] = $result;

                    if ($attributes_data) {
                        $current[$tag][$repeated_tag_index[$tag . '_' . $level] . '_attr'] = $attributes_data;
                    }
                    $repeated_tag_index[$tag . '_' . $level]++;

                } else {
                    $current[$tag] = array($current[$tag], $result);
                    $repeated_tag_index[$tag . '_' . $level] = 1;
                    if (isset($current[$tag . '_attr'])) {

                        $current[$tag]['0_attr'] = $current[$tag . '_attr'];
                        unset($current[$tag . '_attr']);
                    }

                    if ($attributes_data) {
                        $current[$tag][$repeated_tag_index[$tag . '_' . $level] . '_attr'] = $attributes_data;
                    }
                    $repeated_tag_index[$tag . '_' . $level]++;
                }
            }
        } elseif ($type == 'close') {
            $current = &$parent[$level - 1];
        }
    }
    return ($xml_array);
}

class Variablen
{
    var $returner = null;

    function __construct($var)
    {
        switch ($var['Variable']) {
            case 'PHPVersion':
                $this->returner = $this->check_php_version();
                break;
            case 'MySQLVersion':
                $this->returner = $this->check_mysql_version();
                break;
            case 'MySQLiVorhanden':
                $this->returner = $this->check_mysqli();
                break;
            case 'GZipVorhanden':
                $this->returner = $this->check_zlib();
                break;
            case 'PHPConnectorVersion':
                $this->returner = call_success(array('Inhalt' => OSCWARE_CONNECTOR));
                break;
            default:
                $this->returner = call_error('0001');
                break;
        }
    }

    function check_php_version()
    {
        if ($php = phpversion()) return call_success(array('Inhalt' => $php));
        else return call_error('3011');
    }

    function check_mysql_version()
    {
        $mysql = new DB();
        if ($mysql = $mysql->version()) return call_success(array('Inhalt' => $mysql));
        else return call_error('3021');
    }

    function check_mysqli()
    {
        if ($mysqli = extension_loaded('mysqli')) return call_success(array('Inhalt' => $mysqli));
        else return call_error('3031');;
    }

    function check_zlib()
    {
        if ($zlib = extension_loaded('zlib')) return call_success(array('Inhalt' => $zlib));
        else return call_error('3041');
    }
}

class MySQL
{
    var $workaround = false;
    var $returner = null;
    var $db = null;

    function __construct($var)
    {
        global $dbx;
        if (TRANSAKTION && $dbx) $this->db = $dbx;
        else $this->db = null;

        if (isset($var['Zugangsdaten']) && count($var['Zugangsdaten']) > 0 && $var['Zugangsdaten'] != '') {
            if ($this->db == false) $this->db = new DB($var['Zugangsdaten']['Host'], $var['Zugangsdaten']['Datenbank'], $var['Zugangsdaten']['Benutzer'], $var['Zugangsdaten']['Passwort']);
        } else
            if ($this->db == false) $this->db = new DB();

        if (TRANSAKTION && !$dbx) $dbx = $this->db;
        $check = false;
        $param = array();
        if (isset($var["ParameterListe"]["Parameter"])) {
            if (is_array($var["ParameterListe"]["Parameter"])) {
                $param = $var["ParameterListe"]["Parameter"];
            } else {
                $param = array('0' => $var["ParameterListe"]["Parameter"]);
                if (isset($var["ParameterListe"]["Parameter_attr"])) {
                    $param['0_attr'] = $var["ParameterListe"]["Parameter_attr"];
                }
            }
            $param = $this->parse_parameter($param);
        }
        if ($this->db->connected != false) {
            $insertpos = strpos($var["SQLAusdruck"], "INSERT");
            if ($var["SQLAnfrageTyp"] == "Abfrage" && (($insertpos === false) || ($insertpos !== false && $insertpos > 6))) {
                $query = $var["SQLAusdruck"];

                $check = $this->db->selectquery($query, $param);
                if ($check == 'works') {
                    $parse_content = $this->db->return_result();
                    $parse_header = $this->db->feldliste;
                    $this->returner = $this->generate_request_export($parse_header, $parse_content);
                } else {
                    $this->returner = (!is_array($check)) ? call_error($check) : call_error($check[0], $check[1]);
                }
            } elseif ($var["SQLAnfrageTyp"] == "Abfrage" && $insertpos !== false && $insertpos <= 6) {
                $this->returner = call_error(1006);
            } elseif ($var["SQLAnfrageTyp"] == "Befehl") {
                $query = $var["SQLAusdruck"];
                $check = $this->db->commandquery($query, $param);
                if ($check == 'works') {
                    $parse_content = $this->db->return_result();
                    $this->returner = call_success(array('BetroffeneReihen' => $parse_content));
                } else {
                    $this->returner = (!is_array($check)) ? call_error($check) : call_error($check[0], $check[1]);
                }
            } else {
                $this->returner = call_error(9998);
            }
        } else {
            $this->returner = call_error($this->db->connectederror, $this->db->suberror);
        }
    }

    function parse_parameter($parameter)
    {
        $query_parameter = array();
        $parameter_attribute = array();
        foreach ($parameter as $key => $value) {
            if (str_replace('_attr', '', $key) != $key) {
                $parameter_attribute[$key] = $value;
            } else {
                $query_parameter[$key] = $value;
            }
        }
        foreach ($query_parameter as $key => $value) {
            if (isset($parameter_attribute[$key . '_attr'])) {
                $query_parameter[$key] = (isset($parameter_attribute[$key . '_attr']["isNull"])) ? null : $value;
            } else {
                $query_parameter[$key] = $value;
            }
        }
        return $query_parameter;
    }

    function ftype($key)
    {
        if ($key == 'blob' || $key == '252') {
            return true;
        } else {
            return false;
        }
    }

    function generate_request_export($h, $c)
    {
        $_r = array('Erfolgreich' => cdatal . '1' . cdatar,
            'AnzahlReihen' => cdatal . (int)count($c) . cdatar,
            'SpaltenNamenListe' => '',
            'ReihenListe' => '');
        foreach ($h as $key) {
            $_r['SpaltenNamenListe'] .= '<SpaltenName>' . cdatareplace($key) . '</SpaltenName>';
        }
        foreach ($c as $ckey) {
            $_r['ReihenListe'] .= "<Reihe>\n<SpaltenListe>\n";

            foreach ($ckey as $content) {
                if ($this->ftype($content['type']) && $content['content'] && strlen($content['content']) > 0) {
                    $_r['ReihenListe'] .= '<Spalte base64="1">' . cdatareplace(base64_encode($content['content'])) . '</Spalte>';
                } else {
                    $_r['ReihenListe'] .= '<Spalte>' . cdatareplace($content['content']) . '</Spalte>';
                }

            }
            $_r['ReihenListe'] .= "</SpaltenListe>\n</Reihe>\n";
        }
        return $_r;
    }
}

class DB
{
    var $connection = null;
    var $commiterror = false;
    var $mysqlbegin = false;
    var $result = null;
    var $counter = null;
    var $db = false;
    var $mysqli = 0;
    var $mysql = 0;
    var $feldliste = array();
    var $connected = false;
    var $defaultCharset = null;
    var $charset = null;
    var $connectederror = false;
    var $suberror = false;


    function __construct($host = null, $database = null, $user = null, $pass = null)
    {

        if ($host != null && $database != null && $user != null) {
            $this->db = array('database' => $database, 'host' => $host, 'user' => $user, 'pass' => $pass, 'port' => '');
        } else {
            if (defined("LOCALCONFIG") && LOCALCONFIG == 'ready' && $host == null && $database == null && $user == null && $pass == null) {
                $this->db = array('database' => X_DB_DATABASE, 'host' => X_DB_SERVER, 'user' => X_DB_SERVER_USERNAME, 'pass' => X_DB_SERVER_PASSWORD, 'port' => '');
            }
        }

        if (!$this->db || !$this->db["host"]) {
            $this->connectederror = 1001;
            return;
        }

        if (str_replace(":", "", $this->db["host"]) != $this->db["host"]) {
            $p = explode(":", $this->db["host"]);
            $this->db["host"] = $p[0];
            if (ctype_digit($p[1])) {
                $this->db["port"] = ':' . $p[1];
                ini_set('mysqli.default_port', $p[1]);
            } else {
                ini_set('mysqli.default_socket', $p[1]);
            }

        }
        if ($this->db) {
            $this->mysqli = extension_loaded('mysqli');
            $this->mysql = extension_loaded('mysql');

            try {
                $this->connect();
            } catch (mysqli_sql_exception $e) {
                trigger_error($e, E_USER_WARNING);
                $this->connectederror = 1005;
            }
        } else {
            $this->connectederror = 1005;
        }
    }


    function connect()
    {
        if ($this->mysqli) {
            $this->connection = new mysqli($this->db["host"], $this->db["user"], $this->db["pass"],
                $this->db["database"]);
            if ($this->connection->connect_error) {
                $this->suberror = $this->connection->connect_error;
                $this->connectederror = 'DB' . $this->connection->connect_errno;
            } else {
                $this->connected = true;
                $this->defaultCharset = $this->connection->get_charset()->charset;
                $this->connection->set_charset(MYSQL_CONNECTION_CHARSET);
                $this->connection->query('SET NAMES "'.MYSQL_CONNECTION_CHARSET.'"');
                $this->charset = $this->connection->get_charset()->charset;
            }
        } elseif ($this->mysql) {
            $this->connection = mysql_connect($this->db["host"] . $this->db["port"], $this->db["user"], $this->db["pass"], true);
            if (!$this->connection) {
                $this->suberror = mysql_error();
                $this->connectederror = 'DB' . mysql_errno();
            } else {
                if (!mysql_select_db($this->db["database"], $this->connection)) {
                    $this->suberror = mysql_error();
                    $this->connectederror = 'DB' . mysql_errno($this->connection);
                } else {
                    $this->connected = true;
                    $this->defaultCharset = mysql_client_encoding($this->connection);
                    mysql_set_charset(MYSQL_CONNECTION_CHARSET, $this->connection);
                    $this->charset = mysql_client_encoding($this->connection);
                }
            }
        } else {
            trigger_error('No MySQL extension enabled', E_USER_WARNING);
            $this->connectederror = 'DB0000';
        }
    }

    function disconnect()
    {
        if (is_resource($this->connection)) {
            if($this->mysqli) {
                mysqli_close($this->connection);
            } else {
                mysql_close($this->connection);
            }
        }
    }

    function mysqli_blob($type, $f = false)
    {
        if ($f) {
            switch ($type) {
                case MYSQLI_TYPE_TINY_BLOB:
                case MYSQLI_TYPE_MEDIUM_BLOB:
                case MYSQLI_TYPE_LONG_BLOB:
                case MYSQLI_TYPE_BLOB:
                    return 'blob';
                    break;
            }
        }
        return 'text';
    }


    function getReferences($arr)
    {
        if (strnatcmp(phpversion(), '5.3') >= 0) {
            $refs = array();
            foreach ($arr as $key => $value) {
                $refs[$key] = &$arr[$key];
            }
            return $refs;
        }
        return $arr;
    }

    function selectquery($query, $param = array())
    {
        $this->feldliste = array();
        if ($this->mysqli == 0) {
            $pc = count($param); // anzahl parameter
            $qq = (str_replace('?', '', $query) != $query); // ist der string mit param ? bestückt?
            if (!$qq) {
                if (!$this->result = mysql_query($query, $this->connection)) {
                    $this->suberror = mysql_error();
                    return 'DB' . mysql_errno();
                } else {
                    $this->result = $this->fetcharray();
                    return 'works';
                }
            } else {
                preg_match_all("([\?]{1})", $query, $ausgabe, PREG_PATTERN_ORDER);
                $cppo = count($ausgabe[0]);
                if ($pc <= 0) return 1003;
                elseif ($pc < $cppo) return 1004;
                elseif ($pc > $cppo) return 1002;
                else {
                    $query = str_replace('%', '~~', $query);
                    $query = str_replace('?', '\'%s\'', $query);
                    $params = $this->param_clear($param);
                    $query = vsprintf($query, $params);
                    $query = str_replace('~~', '%', $query);
                    if (!$this->result = mysql_query($query, $this->connection)) {
                        $this->suberror = mysql_error();
                        return 'DB' . mysql_errno();
                    } else {
                        $this->result = $this->fetcharray();
                        return 'works';
                    }
                }
            }
        } else if ($this->mysqli == 1) {

            $stmttest = $this->connection->stmt_init();
            $stmttest->prepare($query);
            if ($stmttest->error)
                return array('DB' . $stmttest->errno, $stmttest->error);
            $paramcount = $stmttest->param_count;
            $stmttest->close();
            if ($paramcount > 0 && count($param) > 0 && $paramcount == count($param)) {
                $param = (array )$param;
                $stmt = $this->connection->stmt_init();
                $stmt->prepare($query);

                $srepeat = str_repeat("s", count($param));
                array_unshift($param, $srepeat);
                call_user_func_array(array($stmt, 'bind_param'), $this->getReferences($param));
                $stmt->execute();

                $stmt->store_result();
                $meta = $stmt->result_metadata();
                $result = $row = $out = array();
                $feldtype = array();
                $i = 0;
                while ($field = $meta->fetch_field()) {
                    $result[$i] = &$row[$field->name];
                    $this->feldliste[$i] = $field->name;
                    $feldtype[$i] = $this->mysqli_blob($field->type, ($field->flags & 16));
                    $i++;
                }
                call_user_func_array(array($stmt, 'bind_result'), $this->getReferences($result));

                $this->result = array();
                while ($stmt->fetch()) {
                    $values = array();
                    foreach ($result as $kb => $valueres) {
                        $values[$kb] = array("type" => $feldtype[$kb], "content" => $valueres);
                    }

                    $this->result[] = $values;
                }
                $stmt->close();
                return 'works';
            } elseif ($paramcount > 0 && count($param) > 0 && $paramcount < count($param)) {
                return 1002;
            } elseif ($paramcount > 0 && count($param) <= 0) {
                return 1003;
            } elseif ($paramcount > 0 && count($param) < $paramcount) {
                return 1004;
            } elseif ($paramcount <= 0) {
                $this->result = $this->connection->query($query, MYSQLI_USE_RESULT);
                if ($this->connection->error) {
                    return array('DB' . $this->connection->errno, $this->connection->error);
                }
                $this->result = $this->fetcharray();
                return 'works';
            }
        }
    }

    function commandquery($query, $param = array())
    {
        global $dbx, $OSCDEBUG;

        if (TRANSAKTION && $this->commiterror == true) {
            return 7777;
        } else {
            $this->feldliste = array();
            if ($this->mysqli == 0) {
                if (!TRANSAKTION || (TRANSAKTION && !$this->mysqlbegin)) {
                    mysql_query('begin', $this->connection);
                    $this->mysqlbegin = true;
                }
                $pc = count($param); // anzahl parameter
                $qq = (str_replace('?', '', $query) != $query); // ist der string mit param ? bestückt?
                if ($qq) {
                    preg_match_all("([\?]{1})", $query, $ausgabe, PREG_PATTERN_ORDER);
                    $cppo = count($ausgabe[0]);
                    if ($pc <= 0) return 1003;
                    elseif ($pc < $cppo) return 1004;
                    elseif ($pc > $cppo) return 1002;
                    else {
                        $query = str_replace('%', '~~', $query);
                        $query = str_replace('?', ']]?[[', $query);
                        $params = $this->param_clear($param);
                        foreach ($params as $param) {
                            if ($param != null)
                                $query = preg_replace('/\]\]\?\[\[/', "'" . $param . "'", $query, 1);
                            else
                                $query = preg_replace('/\]\]\?\[\[/', 'null', $query, 1);
                        }
                        $query = str_replace('~~', '%', $query);
                    }
                }
                if (!mysql_query($query, $this->connection)) {
                    $this->suberror = mysql_error();
                    $erno = mysql_errno();
                    if (!TRANSAKTION) {
                        mysql_query('rollback', $this->connection);
                    } else {
                        $this->commiterror = true;
                    }
                    return array('DB' . $erno, $this->suberror);
                } else {
                    $this->result = mysql_affected_rows();
                    if (!TRANSAKTION) {
                        mysql_query('commit', $this->connection);
                    }
                    return 'works';
                }
            } else {
                $this->connection->autocommit(false);
                $stmt = $this->connection->prepare($query);
                if ($this->connection->error) {
                    $this->suberror = $this->connection->error;
                    $erno = $this->connection->errno;
                    if (!TRANSAKTION) {
                        $this->connection->rollback();
                    } else {
                        $this->commiterror = true;
                    }
                    return array('DB' . $erno, $this->suberror);
                } else {
                    $paramcount = $stmt->param_count;
                    if (($paramcount > 0 && count($param) > 0 && $paramcount == count($param)) || $paramcount <= 0) {
                        if ($paramcount <= 0) {
                        } else {
                            $param = (array)$param;
                            $srepeat = str_repeat("s", count($param));
                            if ($OSCDEBUG) {
                                $params_string = print_r($param, true);
                                trigger_error("Binding params to query: " . $query . "\n\tParameters: " . $params_string, E_USER_WARNING);
                            }
                            $stmt->bind_param($srepeat, ...$param);
                        }
                        $stmt->execute();
                        if ($stmt->error) {
                            $this->suberror = $stmt->error;
                            $erno = $stmt->errno;
                            $stmt->close();

                            if (!TRANSAKTION) $this->connection->rollback();
                            else $this->commiterror = true;

                            return array('DB' . $erno, $this->suberror);
                        } else {
                            $this->result = $stmt->affected_rows;
                            $stmt->close();
                            if (!TRANSAKTION) $this->connection->commit();

                            return 'works';
                        }
                    } elseif ($paramcount > 0 && count($param) > 0 && $paramcount < count($param)) {
                        return 1002;
                    } elseif ($paramcount > 0 && count($param) <= 0) {
                        return 1003;
                    } elseif ($paramcount > 0 && count($param) < $paramcount) {
                        return 1004;
                    }
                }
            }
        }
    }

    function fetcharray()
    {
        $r = array();
        if ($this->mysqli == 0 && $this->result) {
            $i = 0;
            $feldtype = array();
            while ($i < mysql_num_fields($this->result)) {
                $meta = mysql_fetch_field($this->result, $i);
                $this->feldliste[$i] = $meta->name;
                $feldtype[$i] = $meta->type;
                $i++;
            }
            while ($ret = mysql_fetch_array($this->result, MYSQL_NUM)) {
                foreach ($ret as $key => $value) {

                    $ret[$key] = array("type" => $feldtype[$key], "content" => $value);

                }
                $r[] = $ret;

            }
        } else {
            if ($this->mysqli == 1 && $this->result) {
                $i = 0;
                $feldtype = array();
                $finfo = $this->result->fetch_fields();
                foreach ($finfo as $val) {
                    $this->feldliste[$i] = $val->name;
                    $feldtype[$i] = $this->mysqli_blob($val->type, ($val->flags & 16));
                    $i++;
                }
                while ($ret = $this->result->fetch_array(MYSQLI_NUM)) {
                    foreach ($ret as $key => $value) {
                        $ret[$key] = array("type" => $feldtype[$key], "content" => $value);
                    }
                    $r[] = $ret;
                }
                $this->result->close();
            }
        }


        return $r;
    }

    function param_escape($paramvalue)
    {
        $paramvalue = $this->connection->real_escape_string($paramvalue);
        $paramvalue = addslashes($paramvalue);
        return $paramvalue;
    }

    function param_clear($param)
    {
        if (is_array($param)) {
            $c = count($param) - 1;
            for ($i = 0; $i <= $c; $i++) {
                if ($param[$i] != null) {
                    $param[$i] = $this->param_escape(trim($param[$i], '\'"'));
                }
            }
        } else {
            $param = $this->param_escape(trim($param, '\'"'));
        }
        return $param;
    }

    function count()
    {
        if ($this->counter == null && is_resource($this->result)) {
            $this->counter = mysqli_num_rows($this->result);
        }
        return $this->counter;
    }

    function version()
    {
        if (defined("LOCALCONFIG") && LOCALCONFIG == 'ready' && $this->connected) {
            $query = "SELECT VERSION() as ver";
            $this->selectquery($query);
            return $this->result[0][0]['content'];
        } else {
            return 'Keine Datenbankanbindung';
        }
    }

    function return_result($row = false, $line = false)
    {
        if ($row && !$line) {
            return $this->result[(int)$row];
        } elseif ($row && $line) {
            return $this->result[$row][$line];
        } else {

            return $this->result;
        }
    }

    function transaktion_commit()
    {
        if ($this->commiterror == true) {
            if ($this->mysqli == 1) $this->connection->rollback();
            elseif ($this->mysqli == 0) mysql_query('rollback', $this->connection);
        } else {
            if ($this->mysqli == 1) $this->connection->commit();
            elseif ($this->mysqli == 0) mysql_query('commit', $this->connection);
        }
    }
}

class DummyClass
{
    var $currentVar = false;
    var $currentFound = false;

    function __construct($path)
    {
        include($path);
        return $this;
    }

    function get($k = false)
    {
        $this->currentVar = false;
        $this->currentFound = false;
        if (isset($this->$k)) {
            $this->currentVar = $this->$k;
            $this->currentFound = true;
            return true;
        }
        return false;
    }

    function found()
    {
        return $this->currentFound;
    }

    function val()
    {
        return $this->currentVar;
    }
}


class Datei
{
    var $workaround = false;
    var $returner = null;

    function __construct($var)
    {
        switch ($var["DateiAnfrageTyp"]) {
            case 'Auflisten':
                if (isset($var["Verzeichnis"])) {
                    $var["Verzeichnis"] = str_replace("\r\n", "", $var["Verzeichnis"]);
                    $rekursiv = (isset($var["RekursivAuslesen"]) && (int)$var["RekursivAuslesen"] == 1) ? true : false;
                    $this->returner = $this->list_dir($var["Verzeichnis"], $rekursiv);
                } else
                    $this->returner = call_error(2001);
                break;
            case 'DateiAuslesen':
                $var["Pfad"] = ((isset($var["Pfad"])) ? str_replace("\r\n", "", $var["Pfad"]) : false);
                if (isset($var["Pfad"]) && $var["Pfad"] != '[inigetup]' && $var["Pfad"] != '[oxconfk]')
                    $this->returner = $this->read_file($var["Pfad"]);
                elseif (isset($var["Pfad"]) && $var["Pfad"] == '[inigetup]')
                    $this->returner = $this->getini();
                elseif (isset($var["Pfad"]) && $var["Pfad"] == '[oxconfk]')
                    $this->returner = $this->getoxid('oxconfk');
                else
                    $this->returner = call_error(2011);
                break;

            case 'DateiErstellen':
                $var["Pfad"] = ((isset($var["Pfad"])) ? str_replace("\r\n", "", $var["Pfad"]) : false);
                if (isset($var["Pfad"]) && $var["Pfad"] != '[inisetup]' && $var["Pfad"] != '[htaccess]') {
                    $attribute = (isset($var["Inhalt_attr"])) ? $var["Inhalt_attr"] : array();
                    $this->returner = $this->create_file($var["Pfad"], $var["Inhalt"], $attribute);
                } elseif (isset($var["Pfad"]) && $var["Pfad"] == '[inisetup]') {
                    $this->returner = $this->setini($var["Inhalt"]);
                } elseif (isset($var["Pfad"]) && $var["Pfad"] == '[htaccess]') {
                    $this->returner = $this->htaccess($var["Inhalt"]);
                } else {
                    $this->returner = call_error(2011);
                }
                break;
            case 'DateiLoeschen':
                $var["Pfad"] = str_replace("\r\n", "", $var["Pfad"]);
                $realfilename = str_replace(array('../', './', '/'), '', $var["Pfad"]);
                if (in_array($realfilename, array('x_functions.php', 'connector.php', 'connect.php', 'x_classes.php', 'x_fehlercodes.php'))) {
                    $this->returner = call_error(2035, $realfilename);
                } else {
                    if (@unlink($this->parse_path($var["Pfad"])))
                        $this->returner = array('Erfolgreich' => cdatal . '1' . cdatar);
                    else
                        $this->returner = call_error(2036, $var["Pfad"]);
                }
                break;
            case 'AbsolutenPfadAuslesen':

                if (isset($var["RelativerPfad"])) {
                    $var["RelativerPfad"] = str_replace("\r\n", "", $var["RelativerPfad"]);
                    $this->returner = call_success(array('AbsoluterPfad' => realpath($var["RelativerPfad"])));
                } else
                    $this->returner = call_error(2011);

                break;
            case 'DateiKopieren':
                if (!isset($var["QuellPfad"]) && !isset($var["ZielPfad"])) {
                    $this->returner = call_error(2045);
                } elseif (!isset($var["QuellPfad"]) && isset($var["ZielPfad"])) {
                    $this->returner = call_error(2046);
                } elseif (isset($var["QuellPfad"]) && !isset($var["ZielPfad"])) {
                    $this->returner = call_error(2047);
                } else {
                    $var["QuellPfad"] = str_replace("\r\n", "", $var["QuellPfad"]);
                    $var["ZielPfad"] = str_replace("\r\n", "", $var["ZielPfad"]);
                    $this->returner = $this->copyfile($var["QuellPfad"], $var["ZielPfad"]);
                }
                break;
            case 'DateiVerschieben':
                if (!isset($var["QuellPfad"]) && !isset($var["ZielPfad"])) {
                    $this->returner = call_error(2045);
                } elseif (!isset($var["QuellPfad"]) && isset($var["ZielPfad"])) {
                    $this->returner = call_error(2046);
                } elseif (isset($var["QuellPfad"]) && !isset($var["ZielPfad"])) {
                    $this->returner = call_error(2047);
                } else {
                    $var["QuellPfad"] = str_replace("\r\n", "", $var["QuellPfad"]);
                    $var["ZielPfad"] = str_replace("\r\n", "", $var["ZielPfad"]);
                    $this->returner = $this->move_file($var["QuellPfad"], $var["ZielPfad"]);
                }
                break;
            case 'OrdnerErstellen':
                if (isset($var["Verzeichnis"])) {
                    $var["Verzeichnis"] = str_replace("\r\n", "", $var["Verzeichnis"]);
                    $php = substr(phpversion(), 0, 1);
                    $file = $this->parse_path($var["Verzeichnis"]);
                    if (file_exists($file)) {
                        $this->returner = call_error(2052);
                    } else {
                        if ($php < 5) {
                            $exfile = explode("/", $file);
                            $path = '';
                            foreach ($exfile as $_file) {
                                $path .= $_file . '/';
                                if (!file_exists($path)) {
                                    @mkdir($path, 0777);
                                }
                            }
                            if (file_exists($file))
                                $this->returner = array('Erfolgreich' => cdatal . '1' . cdatar);
                            else
                                $this->returner = call_error(2050);
                        } else {
                            if (mkdir($file, 0777, true))
                                $this->returner = array('Erfolgreich' => cdatal . '1' . cdatar);
                            else
                                $this->returner = call_error(2050);
                        }
                    }
                } else {
                    $this->returner = call_error(2051);
                }

                break;
            case 'OrdnerLoeschen':
                if (isset($var["Verzeichnis"])) {
                    $var["Verzeichnis"] = str_replace("\r\n", "", $var["Verzeichnis"]);
                    $dir = $this->parse_path($var["Verzeichnis"]);

                    if (is_dir($dir)) {
                        if (!rmdir($dir)) {
                            $this->recurse_delete($dir);
                            if (file_exists($dir)) {
                                $this->returner = call_error(2054);
                            } else {
                                $this->returner = call_success();
                            }
                        } else
                            $this->returner = call_success();
                    } else {
                        $this->returner = call_error(2053);
                    }
                } else
                    $this->returner = call_error(2051);
                break;
            case 'OrdnerKopieren':
                if (!isset($var["QuellVerzeichnis"]) && !isset($var["ZielVerzeichnis"])) {
                    $this->returner = call_error(2055);
                } elseif (!isset($var["QuellVerzeichnis"]) && isset($var["ZielVerzeichnis"])) {
                    $this->returner = call_error(2056);
                } elseif (isset($var["QuellVerzeichnis"]) && !isset($var["ZielVerzeichnis"])) {
                    $this->returner = call_error(2057);
                } else {
                    $var["QuellVerzeichnis"] = str_replace("\r\n", "", $var["QuellVerzeichnis"]);
                    $var["ZielVerzeichnis"] = str_replace("\r\n", "", $var["ZielVerzeichnis"]);
                    $dir = $this->parse_path($var["QuellVerzeichnis"]);
                    $newdir = $this->parse_path($var["ZielVerzeichnis"]);
                    if (file_exists($dir)) {
                        $this->recurse_copy($dir, $newdir, $newdir);
                        if (file_exists($newdir)) {
                            $this->returner = call_success();
                        } else {
                            $this->returner = call_error(2061, $dir . ' nach ' . $newdir);
                        }

                    } else {
                        $this->returner = call_error(2060, $dir);
                    }
                }
                break;
            case 'OrdnerVerschieben':
                if (!isset($var["QuellVerzeichnis"]) && !isset($var["ZielVerzeichnis"])) {
                    $this->returner = call_error(2055);
                } elseif (!isset($var["QuellVerzeichnis"]) && isset($var["ZielVerzeichnis"])) {
                    $this->returner = call_error(2056);
                } elseif (isset($var["QuellVerzeichnis"]) && !isset($var["ZielVerzeichnis"])) {
                    $this->returner = call_error(2057);
                } else {
                    $var["QuellVerzeichnis"] = str_replace("\r\n", "", $var["QuellVerzeichnis"]);
                    $var["ZielVerzeichnis"] = str_replace("\r\n", "", $var["ZielVerzeichnis"]);
                    $dir = $this->parse_path($var["QuellVerzeichnis"]);
                    $newdir = $this->parse_path($var["ZielVerzeichnis"]);
                    if (file_exists($dir)) {
                        if (!@rename($dir, $newdir)) {
                            $this->recurse_move($dir, $newdir, $newdir);
                            if (file_exists($dir)) {
                                $this->returner = call_error(2061, $dir . ' nach ' . $newdir);
                            } elseif (!file_exists($newdir)) {
                                $this->returner = call_error(2063, $dir . ' nach ' . $newdir);
                            } else {
                                $this->returner = call_success();
                            }
                        } else {
                            if (file_exists($newdir))
                                $this->returner = call_success();
                            else
                                $this->returner = call_error(2062);
                        }

                    } else {
                        $this->returner = call_error(2060, $dir);
                    }
                }
                break;
            case 'BerechtigungenAuslesen':
                if (isset($var["Pfad"])) {
                    $var["Pfad"] = str_replace("\r\n", "", $var["Pfad"]);
                    $Pfad = $this->parse_path($var["Pfad"]);
                    if (file_exists($Pfad)) {
                        $chmod = fileperms($Pfad);
                        if ($chmod) {
                            $chmod = decoct($chmod);
                            $this->returner = call_success(array('Berechtigungen' => substr($chmod, strlen($chmod) - 4)));
                        } else
                            $this->returner = call_error(2013, $Pfad);

                    } else {
                        $this->returner = call_error(2013, $Pfad);
                    }
                } else {
                    $this->returner = call_error(2011);
                }
                break;
            case 'PruefsummeAuslesen':
                if (isset($var["Pfad"])) {
                    $var["Pfad"] = str_replace("\r\n", "", $var["Pfad"]);
                    $Pfad = $this->parse_path($var["Pfad"]);
                    if (file_exists($Pfad)) {
                        if (is_file($Pfad)) {
                            $chmod = md5_file($Pfad);
                            if ($chmod) {
                                $this->returner = call_success(array('Pruefsumme' => $chmod));
                            } else
                                $this->returner = call_error(2013, $Pfad);
                        } else {
                            $this->returner = call_error(2012, $Pfad);
                        }

                    } else {
                        $this->returner = call_error(2013, $Pfad);
                    }
                } else {
                    $this->returner = call_error(2011);
                }
                break;
            case 'BildAnpassen':
                if (!isset($var["QuellPfad"]) && !isset($var["ZielPfad"])) {
                    $this->returner = call_error(2045);
                } elseif (!isset($var["QuellPfad"]) && isset($var["ZielPfad"])) {
                    $this->returner = call_error(2046);
                } elseif (isset($var["QuellPfad"]) && !isset($var["ZielPfad"])) {
                    $this->returner = call_error(2047);
                } else {
                    $var["QuellPfad"] = str_replace("\r\n", "", $var["QuellPfad"]);
                    $var["ZielPfad"] = str_replace("\r\n", "", $var["ZielPfad"]);
                    $var["AufloesungBreite"] = (int)$var["AufloesungBreite"];
                    $var["AufloesungHoehe"] = (int)$var["AufloesungHoehe"];
                    if ($var["AufloesungBreite"] <= 0 && $var["AufloesungBreite"] <= 0) {
                        $this->returner = $this->copyfile($var["QuellPfad"], $var["ZielPfad"]);
                    } else {
                        $this->returner = $this->copyimage($var["QuellPfad"], $var["ZielPfad"], $var["AufloesungBreite"], $var["AufloesungHoehe"]);
                    }
                }
                break;
            default:
                $this->returner = call_error(9998);
                break;
        }
    }

    function move_file($file, $newfile)
    {
        $file = $this->parse_path($file);
        $newfile = $this->parse_path($newfile);
        if (file_exists($file)) {
            if (!rename($file, $newfile)) {
                return call_error(2048, $file . ' nach ' . $newfile);
            } else {
                if (file_exists($newfile))
                    return call_success();
                else
                    return call_error(2044);
            }
        } else {
            return call_error(2049, $file);
        }
    }

    function copyimage($file, $newfile, $AufloesungBreite, $AufloesungHoehe)
    {
        $file = $this->parse_path($file);
        $newfile = $this->parse_path($newfile);
        $e = explode(".", $newfile);
        $newfileEnd = strtolower(array_pop($e));
        if (!in_array($newfileEnd, array('gif', 'jpg', 'jpeg', 'png'))) {
            return call_error(2083, $file);
        }
        if (file_exists($file)) {
            if (($details = getimagesize($file)) !== false) {
                $currentWidth = $details[0];
                $currentHeight = $details[1];
                $img = false;
                switch (strtolower($details["mime"])) {
                    case 'image/gif':
                        $img = imagecreatefromgif($file);
                        break;
                    case 'image/jpeg':
                        $img = imagecreatefromjpeg($file);
                        break;
                    case 'image/png':
                        $img = imagecreatefrompng($file);
                        break;
                    default:
                        return call_error(2082, $file);
                        break;
                }
                // Verhältnis Breite zu Höhe bestimmen

                if ($currentWidth > $AufloesungBreite || $currentHeight > $AufloesungHoehe) {
                    if ($currentWidth > $currentHeight) {
                        $verhaeltnis = $currentWidth / $currentHeight;
                        $AufloesungHoehe = round($AufloesungBreite / $verhaeltnis, 0);
                    } else {
                        $verhaeltnis = $currentHeight / $currentWidth;
                        $AufloesungBreite = round($AufloesungHoehe / $verhaeltnis, 0);
                    }

                    $newImage = imagecreatetruecolor($AufloesungBreite, $AufloesungHoehe); //Thumbnail im Speicher erstellen
                    $transparent = imagecolorallocatealpha( $newImage, 0, 0, 0, 127 );
                    imagefill( $newImage, 0, 0, $transparent );
                    imagecopyresampled($newImage, $img, 0, 0, 0, 0, $AufloesungBreite, $AufloesungHoehe, $currentWidth, $currentHeight);
                } else {
                    $newImage = imagecreatetruecolor($currentWidth, $currentHeight); //Thumbnail im Speicher erstellen
                    $transparent = imagecolorallocatealpha( $newImage, 0, 0, 0, 127 );
                    imagefill( $newImage, 0, 0, $transparent );
                    imagecopyresampled($newImage, $img, 0, 0, 0, 0, $currentWidth, $currentHeight, $currentWidth, $currentHeight);
                }
                switch ($newfileEnd) {
                    case 'gif':
                        $create = imagegif($newImage, $newfile);
                        break;
                    case 'png':
                        imagealphablending($newImage, false);
                        imagesavealpha($newImage, true);
                        $create = imagepng($newImage, $newfile, 0);
                        break;
                    case 'jpg':
                    case 'jpeg':
                        $create = imagejpeg($newImage, $newfile, 100);
                        break;
                }
                imagedestroy($newImage);
                imagedestroy($img);
                if (!$create) {
                    return call_error(2084, $file . ' nach ' . $newfile);
                } else {
                    if (file_exists($newfile))
                        return call_success();
                    else
                        return call_error(2085);
                }
            } else {
                return call_error(2081, $file);
            }

        } else {
            return call_error(2080, $file);
        }
    }

    function copyfile($file, $newfile)
    {
        $file = $this->parse_path($file);
        $newfile = $this->parse_path($newfile);
        if (file_exists($file)) {
            if (!copy($file, $newfile)) {
                return call_error(2048, $file . ' nach ' . $newfile);
            } else {
                if (file_exists($newfile))
                    return call_success();
                else
                    return call_error(2044);
            }
        } else {
            return call_error(2049, $file);
        }
    }

    function getoxid($t = 'oxconfk')
    {
        $root_array = explode('/', dirname(__FILE__));
        $pathway = '';
        $found = false;
        for ($i = 0; $i < sizeof($root_array); $i++) {
            $pathway .= $root_array[$i] . '/';
            if (file_exists($pathway . 'core/' . $t . '.php')) {
                $found = true;
                $pathway = $pathway . 'core/' . $t . '.php';
                break;
            }
        }
        if ($found) {
            $DummyClass = new DummyClass($pathway);
            $DummyClass->get('sConfigKey');
            if ($DummyClass->found()) {
                return call_success(array('Inhalt' => $DummyClass->val()));
            } else {
                return call_error(2061, 'sConfigKey');
            }
        }
        return call_error(2060, $t);
    }

    function getini()
    {
        $dc = dirname(__FILE__);
        $inhalt = '(Connectorverzeichnis):' . $dc;
        $root_array = explode('/', $dc);
        $pathway = '';
        for ($i = 0; $i < sizeof($root_array); $i++) {
            $pathway .= $root_array[$i] . '/';
            if (file_exists($pathway . 'includes/local/configure.php')) {
                $this->type = 'XTC';
                $this->configpath = $pathway . 'includes/local/configure.php';
                break;
            } elseif (file_exists($pathway . 'includes/configure.php')) {
                $this->type = 'XTC';
                $this->configpath = $pathway . 'includes/configure.php';
                break;
            } elseif (file_exists($pathway . 'core/config/configure.php')) {
                $this->type = 'HHG';
                $this->configpath = $pathway . 'core/config/configure.php';
                break;
            }
        }
        $pathway = rtrim($pathway, '/');
        $this->conroot = $dc;
        $this->getdbhost = $this->getdbuser = $this->getdbpass = $this->getdbname = 'unbekannt';
        if ($pathway != $dc) {
            $inhalt .= "\n" . $this->type . "-Struktur: erkannt";
            $inhalt .= "\nShoproot: " . $pathway;
            $inhalt .= "\nConfig: " . $this->configpath;
            $this->xtc = true;
            $this->shoproot = $pathway;
            $this->getdbhost = 'DB_SERVER';
            $this->getdbuser = 'DB_SERVER_USERNAME';
            $this->getdbpass = 'DB_SERVER_PASSWORD';
            $this->getdbname = 'DB_DATABASE';;
        } else {
            $inhalt .= "\nStruktur: nicht erkannt";
        }
        $inhalt .= "\nHost: " . $this->getdbhost;
        $inhalt .= "\nUser: " . $this->getdbuser;
        $inhalt .= "\nPass: " . $this->getdbpass;
        $inhalt .= "\nDB: " . $this->getdbname;

        if (file_exists('oscware-configure.php')) {
            $connector = $this->read_file('./oscware-configure.php');
            $inhalt .= "\n" . 'Aktuelle INI:' . "\n" . substr($connector['Inhalt'], 9, -3);
        }

        return call_success(array('Inhalt' => $inhalt));
    }

    function setini($inhalt)
    {
        if ($inhalt == '[inigetup]') {
            $this->getini();
            $inhalt = "<?php \n";
            if (isset($this->xtc) && $this->xtc == true) {
                $inhalt .= '$configpath = "' . $this->configpath . '";' . "\n";
                $inhalt .= 'if(file_exists($configpath))' . "\n";
                $inhalt .= '{' . "\n";
                $inhalt .= 'require($configpath);' . "\n";
                $inhalt .= '$dbhost = ' . $this->getdbhost . ';' . "\n";
                $inhalt .= '$dbuser = ' . $this->getdbuser . ';' . "\n";
                $inhalt .= '$dbpass = ' . $this->getdbpass . ';' . "\n";
                $inhalt .= '$dbname = ' . $this->getdbname . ';' . "\n";
                $inhalt .= '}' . "\n";
                $inhalt .= '$shoproot = "' . $this->shoproot . '";' . "\n";
            }
            $inhalt .= '$root = "' . $this->conroot . '";' . "\n";
            $inhalt .= "?>";
        }
        @chmod("oscware-connector.php", 0777);
        $creatini = $this->create_file('./oscware-configure.php', $inhalt);
        @chmod("oscware-connector.php", 0444);
        if ($creatini['Erfolgreich'] == cdatal . '1' . cdatar) {
            $creatini['Inhalt'] = "Configuration wurde mit folgendem Inhalt erzeugt: \n";
            $creatini['Inhalt'] .= $inhalt;
        }
        return $creatini;
    }

    function htaccess($content)
    {
        if ($content != "") {
            $_fnh = './.htpasswd';
            touch($_fnh);
            chmod($_fnh, 0777);  // oktal; richtiger Wert von mode
            $wf = fopen($_fnh, "w+");
            if (!fwrite($wf, $content)) {
                return call_error(2044, "htaccess konnte nicht geschrieben werden! Bitte das Verzeichnis auf 777 chmoden!");
            } else {
                fclose($wf);
                chmod($_fnh, 0644);  // oktal; richtiger Wert von mode
                $path = $_SERVER['SCRIPT_FILENAME'];
                $path = str_replace('/oscware-connector.php', '', $path);
                $htaccessinhalt = <<<EOD
ErrorDocument 401 default
AuthType Basic
AuthName "Please enter your ID and password"
AuthUserFile $path/.htpasswd
require valid-user
EOD;
                $_fnh = './.htaccess';
                touch($_fnh);
                chmod($_fnh, 0777);  // oktal; richtiger Wert von mode
                $wf = fopen($_fnh, "w+");
                if (!fwrite($wf, $htaccessinhalt)) {
                    return call_error(2044, "htaccess konnte nicht geschrieben werden! Bitte das Verzeichnis auf 777 chmoden!");
                } else {
                    fclose($wf);
                    chmod($_fnh, 0644);  // oktal; richtiger Wert von mode
                    $creatini['Erfolgreich'] = cdatal . '1' . cdatar;
                    $creatini['Inhalt'] = "htaccess wurde angelegt";
                }
            }
        } else {
            return call_error(2022, "Kein Inhalt, Schutz wird nicht angelegt");
        }
        return $creatini;
    }

    function create_file($file, $content, $attribute = array())
    {
        $path = $this->parse_path($file);

        if (isset($attribute["base64"]) && $attribute["base64"] == "1") {
            $content = base64_decode($content);
        }
        if (($pos = strrpos($path, "/")) !== false) {
            $pathex = substr($path, 0, $pos + 1);
            if (!is_dir($pathex)) {
                mkdir($pathex, 0777, true);
                chmod($pathex, 0777);
            }
        }
        $fileex = explode('.', $path);
        $fileex = strtolower(array_pop($fileex));
        if (!$handle = @fopen($path, 'w+')) {
            return call_error(2021, $file);
        }

        if (fwrite($handle, $content) === FALSE) {
            return call_error(2022, $path);
        }
        fclose($handle);
        chmod($path, 0777);


        return call_success();
    }


    function parse_path($dirfile)
    {
        $newpath = '';
        if (defined('SHOPROOT') && substr($dirfile, 0, 3) != '../' && substr($dirfile, 0, 2) != './')
            $newpath = SHOPROOT . '/' . trim($dirfile, '/');
        else
            $newpath = rtrim($dirfile, '/');

        return $newpath;
    }


    function list_subdir($path, $rekursive, $basepath)
    {
        $return = array('dir' => array(), 'file' => array());
        if ($dh = opendir($path)) {
            while (($file = readdir($dh)) !== false) {
                if ($file != '.' && $file != '..') {
                    if (filetype($path . '/' . $file) == 'dir') {
                        $return['dir'][] = $basepath . '/' . $file;
                        if ($rekursive && $sub = $this->list_subdir($path . '/' . $file, $rekursive, $basepath . '/' . $file)) {
                            if (count($sub['file']) > 0) {
                                $return['file'] = array_merge($return['file'], $sub['file']);
                            }
                            if (count($sub['dir']) > 0) {
                                $return['dir'] = array_merge($return['dir'], $sub['dir']);
                            }
                        }
                    } elseif (filetype($path . '/' . $file) == 'file') {
                        $return['file'][] = $basepath . '/' . $file;
                    }
                }
            }
            closedir($dh);
        }
        return $return;
    }

    function list_dir($path, $rekursive = false)
    {

        $path = $this->parse_path($path);

        if ($this->check_path_file($path, 'tf', '200', 'dir')) {
            if ($dh = opendir($path)) {
                $_return = array('Erfolgreich' => cdatal . '1' . cdatar, 'OrdnerListe' => '', 'DateienListe' => '');
                while (($file = readdir($dh)) !== false) {
                    if ($file != '.' && $file != '..') {
                        if (filetype($path . '/' . $file) == 'dir') {
                            $_return['OrdnerListe'] .= '<Ordner>' . cdatareplace($file) . '</Ordner>';
                            if ($rekursive && $subs = $this->list_subdir($path . '/' . $file, $rekursive, $file)) {
                                if (count($subs) > 0) {
                                    if (count($subs['dir']) > 0) {
                                        foreach ($subs['dir'] as $subdir) {
                                            $_return['OrdnerListe'] .= '<Ordner>' . cdatareplace($subdir) . '</Ordner>';
                                        }
                                    }
                                    if (count($subs['file']) > 0) {
                                        foreach ($subs['file'] as $subdir) {
                                            $_return['DateienListe'] .= '<Datei>' . cdatareplace($subdir) . '</Datei>';
                                        }
                                    }
                                }
                            }
                        } elseif (filetype($path . '/' . $file) == 'file') {
                            $_return['DateienListe'] .= '<Datei>' . cdatareplace($file) . '</Datei>';
                        }
                    }
                }
                closedir($dh);
            } else {
                $_return = call_error(2004);
            }
        } else {
            $_return = $this->check_path_file($path, 'error', '200', 'dir');
        }
        return $_return;
    }

    function read_file($path)
    {
        $path = $this->parse_path($path);
        if ($this->check_path_file($path)) {
            if ($handle = fopen($path, "rb")) {

                $contents = '';
                while (!feof($handle)) {
                    $contents .= fread($handle, 8192);
                }
                fclose($handle);
                if (@getimagesize($path) != false) {
                    $_return = call_success(array('Inhalt' => base64_encode($contents)));
                } else {
                    $_return = call_success(array('Inhalt' => $contents));
                }

            }

        } else {
            $_return = $this->check_path_file($path, 'error', '201');
        }
        return $_return;
    }


    function check_path_file($path, $returntype = 'tf', $errorbase = '', $neededtype = 'file')
    {

        $neededtype = ($neededtype == 'file') ? 'file' : 'dir';
        if (file_exists($path) && filetype($path) == $neededtype) {
            return true;
        } elseif (file_exists($path) && !filetype($path) == $neededtype) {
            if ($returntype == 'tf') {
                return false;
            }
            $_return = call_error($errorbase . '2', 'Pfad: ' . $path);
        } else {
            if ($returntype == 'tf') {
                return false;
            }
            $_return = call_error($errorbase . '3', 'Pfad: ' . $path);
        }
        return $_return;

    }

    function recurse_copy($src, $dst, $globaldest)
    {
        $dir = opendir($src);
        mkdir($dst);
        while (false !== ($file = readdir($dir))) {
            if (($file != '.') && ($file != '..')) {
                $sourcefile = $src . '/' . $file;
                if ($sourcefile != $globaldest) {
                    if (is_dir($sourcefile)) {
                        $this->recurse_copy($sourcefile, $dst . '/' . $file, $globaldest);
                    } else {
                        copy($sourcefile, $dst . '/' . $file);
                    }
                }
            }
        }
        closedir($dir);
    }

    function recurse_move($src, $dst, $globaldest)
    {
        $dir = opendir($src);
        mkdir($dst);
        while (false !== ($file = readdir($dir))) {
            if (($file != '.') && ($file != '..')) {
                $srcfile = $src . '/' . $file;
                if ($srcfile != $globaldest) {
                    if (is_dir($srcfile)) {
                        $this->recurse_move($src . '/' . $file, $dst . '/' . $file, $globaldest);
                    } else {
                        rename($srcfile, $dst . '/' . $file);
                    }
                }
            }
        }
        closedir($dir);
        rmdir($src);
    }

    function recurse_delete($sourcedir)
    {
        $dir = opendir($sourcedir);
        while (false !== ($file = readdir($dir))) {
            if (($file != '.') && ($file != '..')) {
                $srcfile = $sourcedir . '/' . $file;
                if (is_dir($srcfile)) {
                    $this->recurse_delete($srcfile);
                } else {
                    unlink($srcfile);
                }
            }
        }
        closedir($dir);
        rmdir($sourcedir);
    }
}

$fehlercodes = array(
    '0000' => 'Kein Fehler sondern Returnwert wenn Abfrage funktioniert',
    '0001' => 'unbekannter Aufruf',
    '1001' => 'keine Datenbank-Verbindung',
    '1002' => 'Parameterfehler, mehr Parameter als Platzhalter',
    '1003' => 'Parameterfehler, Platzhalter aber keine Parameter',
    '1004' => 'Parameterfehler, mehr Platzhalter als Parameter',
    '1005' => 'Fehlerhafte Datenbankzugangsdaten',
    '1006' => 'SQLAnfrageTyp Abfrage aber INSERT gegeben',
    '2001' => 'Keine Verzeichnisangabe',
    '2002' => 'Fehlerhafte Pfadangabe: kein Verzeichnis',
    '2003' => 'Fehlerhafte Pfadangabe: Verzeichnis existiert nicht',
    '2004' => 'Verzeichnis nicht lesbar',
    '2011' => 'Keine Dateiangabe',
    '2012' => 'Fehlerhafte Pfadangabe: keine Datei',
    '2013' => 'Fehlerhafte Pfadangabe: Datei existiert nicht',
    '2014' => 'Datei nicht lesbar',
    '2021' => 'Datei nicht lesbar (evtl. Verzeichnisfehler)',
    '2022' => 'Datei Inhalt konnte nicht eingetragen werden',
    '2023' => 'CHMOD kann nicht ausgelesen werden',
    '2035' => 'Systemdate kann nicht entfernt werden',
    '2036' => 'Fehler beim entfernen',
    '2044' => 'Zieldatei wurde nicht angelegt',
    '2045' => 'Pfadangaben fehlen',
    '2046' => 'Quellpfad fehlt',
    '2047' => 'Zielpfad fehlt',
    '2048' => 'Kopieren fehlgeschlagen',
    '2049' => 'Quelldatei existiert nicht',
    '2058' => 'Verschieben fehlgeschlagen',
    '2060' => 'OXID: Datei nicht gefunden',
    '2061' => 'OXID: Key nicht gefunden',
    '2080' => 'Ursprungsbild nicht gefunden',
    '2081' => 'Ursprungsbild nicht als Bild erkannt',
    '2082' => 'Ursprungsbild nicht im Format image/gif,image/jpeg oder image/png',
    '2083' => 'Zielbild nicht im Format image/gif,image/jpeg oder image/png (fehlerhafte Dateiendung)',
    '2084' => 'Bild konnte nicht kopiert werden',
    '2085' => 'Bild konnte nicht erstellt werden',
    '3011' => 'PHP Check Error',
    '3021' => 'MYSQL Check Error',
    '3031' => 'MYSQLi Check Error',
    '3041' => 'Gzip Check Error',
    '7777' => 'Rollback, Vorgang wurde abgebrochen',
    '9998' => 'Unbekannter Befehl',
    '9999' => 'Returner Error',
    'DB0000' => 'Keine PHP MySQL Erweiterung aktiviert',
    'DB1045' => 'Fehler bei Datennbankzugriff (Benutzer/Passwortfehler)',
    'DB1044' => 'Keine Berechtigung bei Datenbankzugriff (Falsche Datenbank?)',
    'DB1054' => 'Unbekannte Spalte innerhalb der Anfrage',
    'DB1064' => 'Fehler im Aufbau der Abfrage',
    'DB1146' => 'Tabelle existiert nicht'
);

$dbx = $suberror = false;
$xml = $data = array();
$i = 0;

if (file_exists('oscware-configure.php')) {
    $shoproot = null;
    $dbhost = null;
    $dbuser = null;
    $dbpass = null;
    $dbname = null;
    $oscdebug = false;

    require_once('oscware-configure.php');

    if ($oscdebug) {
        $OSCDEBUG = true;
    }

    if ($shoproot) {
        $shoproot = rtrim($shoproot, '/');
    }

    define('LOCALCONFIG', 'ready');
    define('SHOPROOT', $shoproot);
    define('X_DB_SERVER', $dbhost);
    define('X_DB_SERVER_USERNAME', $dbuser);
    define('X_DB_SERVER_PASSWORD', $dbpass);
    define('X_DB_DATABASE', $dbname);
    define("CONFIGFOWNER", fileowner('oscware-configure.php'));
    define("CONFIGFGROUP", filegroup('oscware-configure.php'));
} else {
    define("CONFIGFOWNER", false);
    define("CONFIGFGROUP", false);
}

if ($postText = file_get_contents('php://input')) {
    if ($OSCDEBUG) {
        trigger_error('Request: "' . str_replace("\n", '\n', $postText) . '"');
    }
    //$postText = preg_replace('|>\s*<|', ">\n<", $postText);

    $xml = xml2array($postText);
    $xmlroot = $xml["PHPConnector"];

    if ($xmlroot['Transaktion'] == 1)
        define('TRANSAKTION', true);
    else
        define('TRANSAKTION', false);

    if ($xmlroot['Kompression'] == 1)
        define('KOMPRESSION', true);
    else
        define('KOMPRESSION', false);

    if (isset($xmlroot['Debug']) && $xmlroot['Debug'] == 1)
        $OSCDEBUG = true;

    if ($OSCDEBUG) {
        // E_USER is used for custom messages that should be logged, don't display them
        @error_reporting(E_ALL ^ (E_USER_NOTICE | E_USER_WARNING | E_USER_ERROR));
        @ini_set('display_errors', 1);
    }

    $AnfragenListe = $xmlroot['AnfragenListe'];
    if (!isset($AnfragenListe["Anfrage"][0]) || !is_array($AnfragenListe["Anfrage"][0])) {
        $new = $AnfragenListe["Anfrage"];
        unset($AnfragenListe["Anfrage"]);
        $AnfragenListe["Anfrage"][] = $new;
    }

    foreach ($AnfragenListe["Anfrage"] as $key => $value) {
        switch ($value['Typ']) {
            case 'Variablen':
            case 'MySQL':
            case 'Datei':
                $av = new $value['Typ']($value);
                break;
            default:
                break;
        }
        if ($av) {
            if ($av->returner)
                $data[] = array('ID' => $value['ID'], 'Antwort' => $av->returner);
            else
                $data[] = array('ID' => $value['ID'], 'Antwort' => call_error('9999'));
        }
    }

    if (TRANSAKTION && $dbx)
        $dbx->transaktion_commit();

    $build = build_xml($data);

    if ($OSCDEBUG) {
        trigger_error('Response: "' . str_replace("\n", '\n', $build) . '"');
    }

    // Check if the output should be compressed and whether the content-encoding header
    // should be set. OscWare does it's own decompression independent of the HTTP stack
    // so setting the parameter without support in OscWare causes errors.
    $setContentEncoding = array_key_exists('HTTP_ACCEPT_ENCODING', $_SERVER) &&
        strpos($_SERVER['HTTP_ACCEPT_ENCODING'], 'gzip');
    if (KOMPRESSION || $setContentEncoding) {
        $build = gzencode($build);
    }

    // Discard all previous output and log a warning if there is any
    $contents = ob_get_contents();
    if ($contents && strlen($contents) > 0) {
        trigger_error('Pre-XML Response: "' . str_replace("\n", '\n', $contents) . '"', E_USER_WARNING);
    }
    ob_clean();

    header('Content-Type: text/xml');
    if ($setContentEncoding) {
        header('Content-Encoding: gzip');
    }

    echo $build;
} else {
    $db = new DB();
    echo 'OscWare-Connector Version: ' . OSCWARE_CONNECTOR . '<br />';
    echo 'PHP Version: ' . phpversion() . '<br />';
    echo 'MySQL Version: ' . $db->version() . '<br />';
    echo 'MySQLi Vorhanden: ' . (extension_loaded('mysqli') ? '1' : '0') . '<br />';
    echo 'GZip Vorhanden: ' . (extension_loaded('zlib') ? '1' : '0') . '<br />';
    echo 'MySQL default charset: ' . $db->defaultCharset . '<br />';
    echo 'MySQL current charset: ' . $db->charset . '<br />';
    echo 'Debug mode: ' . ($OSCDEBUG ? 'true' : 'false') . '<br />';
}

$contentLength = ob_get_length();
ob_end_flush();

$ip = $_SERVER['REMOTE_ADDR'];
$method = $_SERVER['REQUEST_METHOD'];

$query = '';
if (array_key_exists('QUERY_STRING', $_SERVER)) {
    $query = '?'.$_SERVER['QUERY_STRING'];
}

$url = $_SERVER['SCRIPT_FILENAME'] . $query;
$ua = $_SERVER['HTTP_USER_AGENT'];

if ($OSCDEBUG) {
    trigger_error("Request done - $ip - \"$method $url\" 200 $contentLength \"-\" \"$ua\"");
}

$DONE = true;
